// ====================================================================
// Copyright (C) 2018 - Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================
// *****************************************************************
// max() for DD
//
// Calling Sequence :
// max(DD)
// max(DD, "r"|1)
// max(DD, "c"|2)
//
// Authors :
// Copyright (C) 2018 - Samuel GOUGEON
// last updating : 2018-07-31
// *****************************************************************
function [v, K] = ddqd_minmax(minmax, A, opt)
    // internal called by %dd_min(), %qd_min(), %dd_max(), %qd_max()

    // Initializations
    cancel = -%inf          // value to cancel unrelevant values
    if typeof(A)=="dd" then
        pmax = 3    // total number of tlist elements
    else
        pmax = 5
    end

    // Arguments checking
    if ~isdef("opt","l") then
        opt = "*"
    else
        if and(type(opt)~=[1 10]) | ~(or(opt==[1 2]) | or(opt==["r" "c"]))
            msg = _("%s: Argument #%d: Must be in the set {%s}.\n")
            error(msprintf(msg, "%"+typeof(A)+"_"+minmax, 2, "''r'',1,''c'',2"))
        end
        if opt==1
            opt = "r"
        elseif opt==2
            opt = "c"
        end
    end

    // Pre-processing
    transpose = %f
    if opt=="c" then
        opt = "r"
        transpose = %t
        A = A'
    elseif opt=="*"
        A = A(:,0)  // equivalent to A(:) (but works)
    end

        // for min()
    if minmax=="min"
        A = -A
    end
        // Ignoring %nan
    for p = 2:pmax
        tmp = isnan(A(p));
        A(p)(tmp) = cancel;
    end

    // Processing
    s = size(A(2))
    K = zeros(1,s(2))
    canceled = zeros(s(1),s(2))
    for p = 2:pmax
        a = A(p)
        [m, k] = max(a+canceled,"r")
        // Are there several max on each column?
        boo = a==(ones(s(1),1) * m)
        nmc = sum(boo, "r"); // Number of Max on Columns
        tmp = K==0 & nmc==1
        K(tmp) = k(tmp)
        if and(K>0)
            break
        end
        canceled(~boo) = canceled(~boo) + cancel
    end
    tmp = K==0
    K(tmp) = k(tmp)
        // Get values on k
    tmp = sub2ind(s, K', (1:s(2))');
    v = A(tmp,0)'

    // Post-processing
    if transpose then
        v = v'
        K = K'
    end
    if minmax=="min" then
        v = -v
    end
endfunction
