// ====================================================================
// Copyright (C) 2018 - Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================
// *****************************************************************
// max() for DD
//
// Calling Sequence :
// max(DD)
// max(DD, "r"|1)
// max(DD, "c"|2)
//
//
// Authors :
// Copyright (C) 2018 - Samuel GOUGEON
// last updating : 2018-07-31
// *****************************************************************
function [v, K] = %dd_max(A, opt)

    // Initializations
    if typeof(A)=="dd" then
        pmax = 3    // total number of tlist elements
    else
        pmax = 5
    end

    // Arguments checking
    if ~isdef("opt","l") then
        opt = "*"
    else
        if and(type(opt)~=[1 10]) | ~(or(opt==[1 2]) | or(opt==["r" "c"]))
            msg = _("%s: Argument #%d: Must be in the set {%s}.\n")
            error(msprintf(msg, "%dd_max", 2, "''r'',1,''c'',2"))
        end
        if opt==1
            opt = "r"
        elseif opt==2
            opt = "c"
        end
    end

    // Pre-processing
    transpose = %f
    if opt=="c" then
        opt = "r"
        transpose = %t
        A = A'
    elseif opt=="*"
        for p = 2:pmax
            A(p) = A(p)(:)
        end
    end
        // Ignoring %nan
    for p = 2:pmax
        tmp = isnan(A(p));
        A(p)(tmp) = -%inf;
    end

    // Processing
    s = size(A(2))
    K = zeros(1,s(2))
    for p = 2:pmax
        a = A(p)
        [m, k] = max(a,"r")
        // Are there several max on each column?
        nmc = sum(a==(ones(s(1),1) * m), "r"); // Number of Max on Columns
        tmp = K==0 & nmc==1
        K(tmp) = k(tmp)
        if and(K>0)
            break
        end
    end
    tmp = K==0
    K(tmp) = k(tmp)
        // Get values on k
    tmp = sub2ind(s, K', (1:s(2))');
    v = A(tmp,0)'

    // Post-processing
    if transpose then
        v = v'
        K = K'
    end
endfunction
