// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// QD multiplication (qd * qd)
//
// Calling Sequence :
// a * b
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// last updating : 2011/09/12 
// *****************************************************************
function c = %qd_m_qd(a,b)
    [m1,n1] = size(a.d);
    [m2,n2] = size(b.d);
    //(1,1)*(1,1),(1,1)*(m,n),(m,n)*(1,1)
    if (m1 == 1 & n1 == 1) | (m2 == 1 & n2 == 1) then
        [p1,e1] = two_prod(a(2),b(2));
        [p2,e2] = two_prod(a(2),b(3));
        [p3,e3] = two_prod(a(3),b(2));
        [p4,e4] = two_prod(a(2),b(4));
        [p5,e5] = two_prod(a(3),b(3));
        [p6,e6] = two_prod(a(4),b(2));
        //O(eps) terms
        [p2,p3,e1] = three_sum(p2,p3,e1);
        //O(eps^2) terms
        [p4,p5,p6] = three_sum(p4,p5,p6);
        [p3,e2,e3] = three_sum(p3,e2,e3);
        [p3,p4] = two_sum(p3,p4);
        [p5,e2] = two_sum(e2,p5);
        [p4,p5] = two_sum(p4,p5);
        p5 = p5+e2+e3+p6;
        //O(eps^3) terms
        p4 = p4+a(2)*b(5)+a(3)*b(4)+a(4)*b(3)+a(5)*b(2)+e4+e5+e6; //11
        [t1,t2,t3,t4] = renormalize(p1,p2,p3,p4,p5);
        c = tlist(['qd','d'],t1,t2,t3,t4);

        //(m,n)*(n,1)
    elseif n1 == m2 & n2 == 1 then
        [p1,e1] = two_prod(a(2)(:,1),b(2)(1,1));
        [p2,e2] = two_prod(a(2)(:,1),b(3)(1,1));
        [p3,e3] = two_prod(a(3)(:,1),b(2)(1,1));
        [p4,e4] = two_prod(a(2)(:,1),b(4)(1,1));
        [p5,e5] = two_prod(a(3)(:,1),b(3)(1,1));
        [p6,e6] = two_prod(a(4)(:,1),b(2)(1,1));
        //O(eps) terms
        [p2,p3,e1] = three_sum(p2,p3,e1);
        //O(eps^2) terms
        [p4,p5,p6] = three_sum(p4,p5,p6);
        [p3,e2,e3] = three_sum(p3,e2,e3);
        [p3,p4] = two_sum(p3,p4);
        [p5,e2] = two_sum(e2,p5);
        [p4,p5] = two_sum(p4,p5);
        p5 = p5+e2+e3+p6;
        //O(eps^3) terms
        p4 = p4+a(2)(:,1)*b(5)(1,1)+a(3)(:,1)*b(4)(1,1)+a(4)(:,1)*b(3)(1,1)+a(5)(:,1)*b(2)(1,1)+e4+e5+e6;
        [t1,t2,t3,t4] = renormalize(p1,p2,p3,p4,p5);
        c = tlist(['qd','d'],t1,t2,t3,t4);

        for k=2:n1
            [p1,e1] = two_prod(a(2)(:,k),b(2)(k,1));
            [p2,e2] = two_prod(a(2)(:,k),b(3)(k,1));
            [p3,e3] = two_prod(a(3)(:,k),b(2)(k,1));
            [p4,e4] = two_prod(a(2)(:,k),b(4)(k,1));
            [p5,e5] = two_prod(a(3)(:,k),b(3)(k,1));
            [p6,e6] = two_prod(a(4)(:,k),b(2)(k,1));
            //O(eps) terms
            [p2,p3,e1] = three_sum(p2,p3,e1);
            //O(eps^2) terms
            [p4,p5,p6] = three_sum(p4,p5,p6);
            [p3,e2,e3] = three_sum(p3,e2,e3);
            [p3,p4] = two_sum(p3,p4);
            [p5,e2] = two_sum(e2,p5);
            [p4,p5] = two_sum(p4,p5);
            p5 = p5+e2+e3+p6;
            //O(eps^3) terms
            p4 = p4+a(2)(:,k)*b(5)(k,1)+a(3)(:,k)*b(4)(k,1)+a(4)(:,k)*b(3)(k,1)+a(5)(:,k)*b(2)(k,1)+e4+e5+e6;
            [t1,t2,t3,t4] = renormalize(p1,p2,p3,p4,p5);
            d = tlist(['qd','d'],t1,t2,t3,t4);
            c = %qd_a_qd(c,d);
        end
        //(1,l)*(l,n)
    elseif m1 == 1 & n1 == m2 then
        [p1,e1] = two_prod(a(2)(1,1),b(2)(1,:));
        [p2,e2] = two_prod(a(2)(1,1),b(3)(1,:));
        [p3,e3] = two_prod(a(3)(1,1),b(2)(1,:));
        [p4,e4] = two_prod(a(2)(1,1),b(4)(1,:));
        [p5,e5] = two_prod(a(3)(1,1),b(3)(1,:));
        [p6,e6] = two_prod(a(4)(1,1),b(2)(1,:));
        //O(eps) terms
        [p2,p3,e1] = three_sum(p2,p3,e1);
        //O(eps^2) terms
        [p4,p5,p6] = three_sum(p4,p5,p6);
        [p3,e2,e3] = three_sum(p3,e2,e3);
        [p3,p4] = two_sum(p3,p4);
        [p5,e2] = two_sum(e2,p5);
        [p4,p5] = two_sum(p4,p5);
        p5 = p5+e2+e3+p6;
        //O(eps^3) terms
        p4 = p4+a(2)(1,1)*b(5)(1,:)+a(3)(1,1)*b(4)(1,:)+a(4)(1,1)*b(3)(1,:)+a(5)(1,1)*b(2)(1,:)+e4+e5+e6;
        [t1,t2,t3,t4] = renormalize(p1,p2,p3,p4,p5);
        c = tlist(['qd','d'],t1,t2,t3,t4);

        for k=2:m2
            [p1,e1] = two_prod(a(2)(1,k),b(2)(k,:));
            [p2,e2] = two_prod(a(2)(1,k),b(3)(k,:));
            [p3,e3] = two_prod(a(3)(1,k),b(2)(k,:));
            [p4,e4] = two_prod(a(2)(1,k),b(4)(k,:));
            [p5,e5] = two_prod(a(3)(1,k),b(3)(k,:));
            [p6,e6] = two_prod(a(4)(1,k),b(2)(k,:));
            //O(eps) terms
            [p2,p3,e1] = three_sum(p2,p3,e1);
            //O(eps^2) terms
            [p4,p5,p6] = three_sum(p4,p5,p6);
            [p3,e2,e3] = three_sum(p3,e2,e3);
            [p3,p4] = two_sum(p3,p4);
            [p5,e2] = two_sum(e2,p5);
            [p4,p5] = two_sum(p4,p5);
            p5 = p5+e2+e3+p6;
            //O(eps^3) terms
            p4 = p4+a(2)(1,k)*b(5)(k,:)+a(3)(1,k)*b(4)(k,:)+a(4)(1,k)*b(3)(k,:)+a(5)(1,k)*b(2)(k,:)+e4+e5+e6;
            [t1,t2,t3,t4] = renormalize(p1,p2,p3,p4,p5);
            d = tlist(['qd','d'],t1,t2,t3,t4);
            c = %qd_a_qd(c,d);
        end
        //(m,l)*(l,n)
    elseif n1 == m2 then
        C = zeros(m1,n2);
        c = tlist(['qd','d'],C,C,C,C);
        for i=1:m1
            [p1,e1] = two_prod(a(2)(i,1),b(2)(1,:));
            [p2,e2] = two_prod(a(2)(i,1),b(3)(1,:));
            [p3,e3] = two_prod(a(3)(i,1),b(2)(1,:));
            [p4,e4] = two_prod(a(2)(i,1),b(4)(1,:));
            [p5,e5] = two_prod(a(3)(i,1),b(3)(1,:));
            [p6,e6] = two_prod(a(4)(i,1),b(2)(1,:));
            //O(eps) terms
            [p2,p3,e1] = three_sum(p2,p3,e1);
            //O(eps^2) terms
            [p4,p5,p6] = three_sum(p4,p5,p6);
            [p3,e2,e3] = three_sum(p3,e2,e3);
            [p3,p4] = two_sum(p3,p4);
            [p5,e2] = two_sum(e2,p5);
            [p4,p5] = two_sum(p4,p5);
            p5 = p5+e2+e3+p6;
            //O(eps^3) terms
            p4 = p4+a(2)(i,1)*b(5)(1,:)+a(3)(i,1)*b(4)(1,:)+a(4)(i,1)*b(3)(1,:)+a(5)(i,1)*b(2)(1,:)+e4+e5+e6;
            [t1,t2,t3,t4] = renormalize(p1,p2,p3,p4,p5);
            d = tlist(['qd','d'],t1,t2,t3,t4);
            for k=2:n1
                [p1,e1] = two_prod(a(2)(i,k),b(2)(k,:));
                [p2,e2] = two_prod(a(2)(i,k),b(3)(k,:));
                [p3,e3] = two_prod(a(3)(i,k),b(2)(k,:));
                [p4,e4] = two_prod(a(2)(i,k),b(4)(k,:));
                [p5,e5] = two_prod(a(3)(i,k),b(3)(k,:));
                [p6,e6] = two_prod(a(4)(i,k),b(2)(k,:));
                //O(eps) terms
                [p2,p3,e1] = three_sum(p2,p3,e1);
                //O(eps^2) terms
                [p4,p5,p6] = three_sum(p4,p5,p6);
                [p3,e2,e3] = three_sum(p3,e2,e3);
                [p3,p4] = two_sum(p3,p4);
                [p5,e2] = two_sum(e2,p5);
                [p4,p5] = two_sum(p4,p5);
                p5 = p5+e2+e3+p6;
                //O(eps^3) terms
                p4 = p4+a(2)(i,k)*b(5)(k,:)+a(3)(i,k)*b(4)(k,:)+a(4)(i,k)*b(3)(k,:)+a(5)(i,k)*b(2)(k,:)+e4+e5+e6;
                [t1,t2,t3,t4] = renormalize(p1,p2,p3,p4,p5);
                e = tlist(['qd','d'],t1,t2,t3,t4);
                d = %qd_a_qd(d,e);
            end
            c(i,:) = d;
        end
    else
        error(msprintf(_("%s: Inconsistent multiplication.\n", "%qd_m_qd")))
    end
endfunction
