// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// QD multiplication (qd * double)
//
// Calling Sequence :
// a * b
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// last updating : 
// *****************************************************************
function c = %qd_m_s(a,b)
    [m1,n1] = size(a.d);
    [m2,n2] = size(b);
    //(1~1)~(1~1),(1~1)~(m~n),(m~n)~(1~1)
    if (m1 == 1 & n1 == 1) | (m2 == 1 & n2 == 1) then
        c = prod_qd_d(a,b)
        //(m~n)~(n~1)
    elseif n1 == m2 & n2 == 1 then
        c = prod_qd_d(a(:,1),b(1,1));
        for k=2:n1
            d = prod_qd_d(a(:,k),b(k,1));
            c = %qd_a_qd(c,d);
        end
        //(1~l)~(l~n)
    elseif m1 == 1 & n2 == m2 then
        c = prod_qd_d(a(1,1),b(1,:));
        for k=2:n1
            d = prod_qd_d(a(1,k),b(k,:));
            c = %qd_a_qd(c,d);
        end
        //(m~l)~(l~n)
    elseif n1 == m2 then
        C = zeros(m1,n2);
        c = tlist(['qd','d'],C,C,C,C);
        for i=1:m1
            d = prod_qd_d(a(i,1),b(1,:));
            for k=2:n1
                e = prod_qd_d(a(i,k),b(k,:));
                d = %qd_a_qd(d,e);
            end
            c(i,:) = d;
        end
    else
        error(msprintf(_("%s: Inconsistent multiplication.\n", "%qd_m_s")))
    end
endfunction


function c = prod_qd_d(a,b)
    //QD multiplication
    //QD * D
    [p1,e1] = two_prod(a(2),b);
    [p2,e2] = two_prod(a(3),b);
    [p3,e3] = two_prod(a(4),b);
    p4 = a(5)*b;
    [p2,e1] = two_sum(p2,e1);
    [p3,e2,e1] = three_sum(p3,e2,e1);
    [p4,e2] = three_sum2(p4,e3,e2); 
    p5 = e1+e2;
    [t1,t2,t3,t4] = renormalize(p1,p2,p3,p4,p5);
    c = tlist(['qd','d'],t1,t2,t3,t4);
endfunction
