// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
// Copyright (C) 2018 - Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// norms of a matrix of QD numbers
//
// Calling Sequence :
// norm(x,.N)
//
// Arguments :
// x : QD matrix (or column, for N=2)
// N : 1 or 2 or %inf or 'fro'
//
// ***TODO : extend norm(*,2) to matrices (restricted to columns)
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// Copyright (C) 2018 - Samuel GOUGEON
// last updating : 2018-07-31
// *****************************************************************
function z = %qd_norm(x,N)

    [m,n] = size(x.d);
    if ~isdef("N","l") then
        N = 2
    elseif type(N)==10
        N = part(convstr(strcat(stripblanks(N))),1)
    end

    // Processing
    // ----------
    select N
    case 1 // 1-norm
        x = abs(x);
        z = 0;
        for i = 1:m
            z = z + x(i,:);
        end
        z = %qd_max(z)

    case 2 // 2-norm
        if n ~= 1 then
            msg = _("%s: Argument #%d: Column expected.\n")
            error(msprintf(msg, "%qd_norm", 1));
        end
        if x == qd(0) then
            z = x
        else
            z = 0;
            for i = 1:m
                z = z + x(i,1)*x(i,1);
            end
            z = sqrt(z);
        end

    case %inf  // infinity-norm
        x = abs(x);
        z = x(:,1)
        for i = 2:n
            z = z + x(:,i)
        end
        z = %qd_max(z)

    case 'f'  // Frobenius
        z = 0;
        for i = 1:m
            tmp =  x(i,:)
            z = z + tmp * tmp'
        end
        z = sqrt(z)

    else
        msg = _("%s: Argument #%d: Must be in the set {%s}.\n")
        error(msprintf(msg, "%qd_norm", 2, "1,2,%inf,''fro''"));
        // z = tlist(['qd','d'],0,0,0,0);
    end
endfunction

