// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
// Copyright (C) 2018 - Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// LU decomposition
//
// Calling Sequence :
// lu(A)
//
// Arguments :
// A : DD or QD matrix
//
// Return :
// L : (n,n) lower triangular matrix
// U : (n,n) upper triangular matrix
//
// make dll file with Visual C++2010
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// Copyright (C) 2018 - Samuel GOUGEON
// last updating : 2018-07-31
// *****************************************************************
//
// INTERNAL called by %dd_lu() and %qd_lu()
function [L,U] = ddqd_lu(A)
    [m,n] = size(A(2))
    if m ~= n then
        msg = _("%s: Argument #%d: Square matrix expected.\n")
        error(msprintf(msg, "lu", 1));
    else
        B = A;
        z = zeros(n,n);
        L = eye(A);
        U = zeros(A);
        for j = 1:n
            for i = 2:j
                  B(i,j) = B(i,j) - B(i,1:(i-1))*B(1:(i-1),j);
            end
            for i = (j+1):n
                k = 1:(j-1)
                if k~=[]
                    B(i,j) = B(i,j) - B(i,k)*B(k,j)
                end
                B(i,j) = B(i,j) / B(j,j);
            end
        end
        for j = 1:n
            U(1:j,j) = B(1:j,j);
            L((j+1):n,j) = B((j+1):n,j);
        end
    end
endfunction
